import java.io.*;
import java.net.*;
import javax.media.*;
import javax.media.rtp.*;
import javax.media.rtp.event.*;
import javax.media.rtp.rtcp.*;
import javax.media.protocol.*;
import javax.media.protocol.DataSource;
import javax.media.format.AudioFormat;
import javax.media.Format;
import javax.media.format.FormatChangeEvent;
import javax.media.control.BufferControl;
import javax.swing.* ;

public class rtpPlayer implements ReceiveStreamListener, ControllerListener
{

	/////////////////////////////////////////////////////////
	/**
    * member variables
	*/
	/////////////////////////////////////////////////////////

    // play session
    private RTPManager m_manager = null;
    private SessionAddress m_serverAddress = null;
    private String m_serverName = null;
    private int m_cport = 0;
    private int m_sport = 0;
    private String m_url = null;
	private Object m_dataSync = new Object() ;
	private boolean m_dataReceived = false ;
    private rtspClient m_client = null;
    private rtspControl m_cmd = null;
    private Player m_player = null;
    private Time m_duration = null;
    private long m_bufferSize = 12000;
    private rtpPlayTimer m_timer = null;
    private float m_volume = 50;
    private String m_error = null;

	/////////////////////////////////////////////////////////
	/**
    * constructor
	*/
	/////////////////////////////////////////////////////////

    public rtpPlayer(rtspClient client)
    {
        m_client = client;
    }

	/////////////////////////////////////////////////////////
	/**
    * setURL
	*/
	/////////////////////////////////////////////////////////

    public void setURL( String url) 
    {
            m_url = url;
            int i=0;
            int start = 0;
            int end = 0;
            int p = m_url.indexOf(':',0);
            p = m_url.indexOf(':',p+1);
            i = m_url.indexOf('/',0);
            start = m_url.indexOf('/',i+1);
            end = m_url.indexOf('/',start+1);
            if ( p == -1) {
                m_serverName = m_url.substring(start+1,end);
            }
            else if ( p != -1 ) {
                m_serverName = m_url.substring(start+1,p);
            }
    }

	/////////////////////////////////////////////////////////
	/**
    * setupSession
	*/
	/////////////////////////////////////////////////////////

    public boolean setupSession()
    {
        try {
            if (m_cmd != null ) {
                m_cmd.closeSocket();
            }

            // setup socket connection to server
            m_cmd = new rtspControl();
            m_cmd.setURL(m_url);
            m_cmd.openSocket();
            m_cmd.generateCSeq();
            m_cmd.generateClientPorts();

            // rtsp describe
            m_cmd.rtspDescribe();
            if ( checkError() == false) {
                return false;
            }

            // rtsp setup 
            m_cmd.rtspPlaySetup();
            if ( checkError() == false) {
                return false;
            }

            // get client/server ports for rtp
            m_cport = m_cmd.getClientPort();
            m_sport = m_cmd.getServerPort();
            m_duration = new Time(m_cmd.getDurationSecs());
        }
        catch ( Exception e ) {
            m_error = "Error: setupSession : " + e.getMessage();
			System.out.println( m_error) ;
            m_client.outputSession(m_error);
            return false;
		}
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * stopSetupSession
	*/
	/////////////////////////////////////////////////////////

    public boolean stopSetupSession()
    {
        try {
            if (teardownSession() == false) {
                return false;
            }
        }
        catch ( Exception e ) {
            m_error = "Error: stopSetupSession : " + e.getMessage();
			System.out.println( m_error) ;
            m_client.outputSession(m_error);
            return false;
		}
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * playSession
	*/
	/////////////////////////////////////////////////////////

    public boolean playSession(String start)
    {
        try {
            createPlaySession();
            setBuffer();
            m_cmd.rtspPlay(start);
            if ( checkError() == false) {
                return false;
            }
            waitForStream();
        }
        catch ( Exception e ) {
            m_error = "Error: playSession : " + e.getMessage();
			System.out.println( m_error) ;
            m_client.outputSession(m_error);
            return false;
		}
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * stopPlaySession
	*/
	/////////////////////////////////////////////////////////

    public boolean stopPlaySession()
    {
        try {
            if ( m_player != null) {
                stopTrackMediaTime();
                m_player.close();
                teardownSession();
            }
        }
        catch ( Exception e ) {
            m_error = "Error: stopPlaySession : " + e.getMessage();
			System.out.println(m_error) ;
            m_client.outputSession(m_error);
            return false;
		}
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * teardownSession
	*/
	/////////////////////////////////////////////////////////

    public boolean teardownSession()
    {
        try {
            m_cmd.rtspTeardown();
            m_cmd.closeSocket();
            if ( m_manager != null) {
                m_manager.removeTarget( m_serverAddress, "Client disconnected");
                m_manager.dispose();
            }
            m_cmd = null;
            m_manager= null;
            m_serverAddress = null;
            m_player = null;
        }
        catch ( Exception ex ) {
            m_error = "Error: teardownSession : " + ex.getMessage();
			System.out.println(m_error) ;
            m_client.outputSession(m_error);
            return false;
		}
        return true;
    }


	/////////////////////////////////////////////////////////
	/**
    * pauseSession
	*/
	/////////////////////////////////////////////////////////

    public boolean pauseSession()
    {
        try {
            if (stopPlaySession()==false) {
                return false;
            }
            if (setupSession()==false) {
                return false;
            }
        }
        catch ( Exception e ) {
            m_error = "Error: pauseSession : " + e.getMessage();
			System.out.println(m_error) ;
            m_client.outputSession(m_error);
            return false;
		}
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * setBuffer
	*/
	/////////////////////////////////////////////////////////

    private void setBuffer()
    {
        try  {
            BufferControl buffer = ( BufferControl ) m_manager.getControl( "javax.media.control.BufferControl" );
            if( buffer != null ) {
                long size = m_bufferSize;
                Double start = new Double(m_client.getStartTime());
                Double duration = new Double(m_duration.getSeconds());
                Double t = new Double(duration.doubleValue() - start.doubleValue());
                long s = (long) (t.doubleValue() * 1000.0);
                if (s < m_bufferSize) {
                    size = s;
                }
                buffer.setBufferLength(s*2);
                //buffer.setMinimumThreshold(s);
            }
        }
        catch ( Exception e ) {
            m_error = "Error: setBuffer : " + e.getMessage();
			System.out.println(m_error) ;
            m_client.outputSession(m_error);
		}
    }
            
	/////////////////////////////////////////////////////////
	/**
    * createPlaySession
	*/
	/////////////////////////////////////////////////////////

    public boolean createPlaySession()
    {
        try {
            InetAddress addr = m_cmd.getLocalAddress();
            String hostAddr = addr.getHostAddress();
            SessionAddress clientAddress = new SessionAddress( InetAddress.getByName(hostAddr), m_cport);
            m_manager = (RTPManager) RTPManager.newInstance() ;
            m_serverAddress = new SessionAddress( InetAddress.getByName( m_serverName ), m_sport ) ;
            m_manager.initialize( clientAddress ) ;
            m_manager.addTarget( m_serverAddress );
            m_manager.addReceiveStreamListener( this ) ;
		}
        catch ( Exception e ) {
            m_error = "Error: createPlaySession : " + e.getMessage();
			System.out.println(m_error) ;
            m_client.outputSession(m_error);
            return false;
		}
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * waitForStream
	*/
	/////////////////////////////////////////////////////////

    public void waitForStream()
    {
		long start = System.currentTimeMillis();
		long duration = 20000;
		try {
    	    synchronized (m_dataSync) {
                while (!m_dataReceived && (System.currentTimeMillis() - start < duration ) ) {
                    m_dataSync.wait(10);
                }
			}
		} 
        catch (Exception e) {
            m_error = "Error in waitForStream : " + e.getMessage();
            m_client.outputSession(m_error);
			System.out.println( m_error ) ;
        }

        if (!m_dataReceived) {
            m_error = "No RTP data was received.";
            System.err.println(m_error);
            m_client.outputSession(m_error);
            m_player.close();
            teardownSession();
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * stopWaitForStream
	*/
	/////////////////////////////////////////////////////////

    private void stopWaitForStream()
    {
        try {
            synchronized( m_dataSync ) {
                m_dataReceived = true ;
                m_dataSync.notifyAll() ;
            }
        }
        catch (Exception e) {
            m_error = "Error in stopWaitForStream :  " + e.toString();
			System.err.println(m_error);
            m_client.outputSession(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * trackMediaTime
	*/
	/////////////////////////////////////////////////////////

    public void trackMediaTime()
    {
		try {
            m_timer = new rtpPlayTimer(m_client, m_player);
            m_timer.start();
		} 
        catch (Exception e) {
            m_error = "Error in trackMediaTime :  " + e.toString();
			System.err.println(m_error);
            m_client.outputSession(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * stopTrackMediaTime
	*/
	/////////////////////////////////////////////////////////

    private void stopTrackMediaTime()
    {
        try {
            if ( m_timer != null) {
                m_timer.setDone();
                m_timer = null;
            }
        }
        catch (Exception e) {
            m_error = "Error in stopTrackMediaTime :  " + e.toString();
			System.err.println(m_error);
            m_client.outputSession(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * startPlaying
	*/
	/////////////////////////////////////////////////////////

    public void startPlaying()
    {
        try {
            GainControl gain = m_player.getGainControl();
            gain.setLevel(m_volume/100);
            Double start = new Double(m_client.getStartTime());
            m_player.setMediaTime( new Time(start.doubleValue()));
            m_player.setStopTime(m_duration);
            m_player.start();
        }
        catch (Exception e) {
            m_error = "Error in startPlaying :  " + e.toString();
			System.err.println(m_error);
            m_client.outputSession(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * controllerUpdate
	*/
	/////////////////////////////////////////////////////////

    public synchronized void controllerUpdate( ControllerEvent event )
	{
		try {
            if( event instanceof RealizeCompleteEvent ) {
                m_client.outputSession("Playing media stream...");
                startPlaying(); 
                trackMediaTime();
            }
            else if ( event instanceof StartEvent) {
                m_client.outputSession("Start Event...");
            }
            else if( event instanceof StopAtTimeEvent ) {
                stopPlaySession();
            }
            else if( event instanceof ControllerClosedEvent ) {
                m_client.stopPlaySession();
            }
        }
        catch (Exception e) {
            m_error = "Error in controllerUpdate :  " + e.toString();
			System.err.println(m_error);
            m_client.outputSession(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * createPlayer
	*/
	/////////////////////////////////////////////////////////

    public void createPlayer( ReceiveStream stream )
    {
        try {
            DataSource ds = stream.getDataSource();
            m_player = javax.media.Manager.createPlayer(ds);
            m_player.addControllerListener( this ) ;
            m_player.realize();
        }
        catch (Exception e) {
            m_error = "Error in createPlayer : " + e.toString();
			System.err.println(m_error);
            m_client.outputSession(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * recieve stream update
	*/
	/////////////////////////////////////////////////////////

    public synchronized void update( ReceiveStreamEvent event) 
    {
        try {
            System.err.println("ReceiveStreamEvent : " + event.toString());
            if ( event instanceof NewReceiveStreamEvent )  {
                m_client.outputSession("New RTP stream received...");
                createPlayer( ((NewReceiveStreamEvent) event ).getReceiveStream());
                stopWaitForStream();
            }
        }
        catch (Exception e) {
            m_error = "Error in ReceiveStreamEvent : " + e.toString();
			System.err.println(m_error);
            m_client.outputSession(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* getDurationSecs
	*/
	/////////////////////////////////////////////////////////

    public double getDurationSecs() 
    {
        return m_cmd.getDurationSecs();
    }

	/////////////////////////////////////////////////////////
	/**
	* setVolume
	*/
	/////////////////////////////////////////////////////////

    public void setVolume(float v) 
    {
        m_volume = v;
        if ( m_player != null) {
            GainControl gain = m_player.getGainControl();
            gain.setLevel(m_volume/100);
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* mute
	*/
	/////////////////////////////////////////////////////////

    public void mute(boolean mute) 
    {
        if ( m_player != null) {
            GainControl gain = m_player.getGainControl();
            if ( mute == true) {
                gain.setLevel(0);
            }
            else {
                gain.setLevel(m_volume/100);
            }
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* checkError
	*/
	/////////////////////////////////////////////////////////

    private boolean checkError()
    {
        boolean res = true;
        try {
            String error = m_cmd.getError();
            if (error != null) {
                m_client.outputSession(error);
                teardownSession();
                res = false;
            }
        }
        catch ( Exception ex) {
			System.err.println("Error in CheckError");
        }
        return res;
    }

}
