import javax.media.*;
 
 public class rtpRecordListener implements javax.media.ControllerListener 
{
	/////////////////////////////////////////////////////////
	/**
    * member variables
	*/
	/////////////////////////////////////////////////////////

    private Player m_player = null;
    private boolean m_configured = false;
    private boolean m_realized = false;
    private boolean m_prefetched = false;
    private boolean m_eom = false;
    private boolean m_failed = false;
    private boolean m_closed = false;
    private rtpRecorder m_recorder = null;
    private float m_volume = 50;
     
	/////////////////////////////////////////////////////////
	/**
    * constructor
	*/
	/////////////////////////////////////////////////////////

    public rtpRecordListener(Player p, rtpRecorder r) 
    {
        m_player = p;
        m_recorder = r;
        m_player.addControllerListener(this);
    }
 
	/////////////////////////////////////////////////////////
	/**
    * configure
	*/
	/////////////////////////////////////////////////////////

    public boolean configure(int timeOutMillis) 
    {
        long startTime = System.currentTimeMillis();
        synchronized (this) {
            if (m_player instanceof Processor) {
                ((Processor)m_player).configure();
            }
            else {
                return false;
            }

            while (!m_configured && !m_failed) {
                try {
                    wait(timeOutMillis);
                }
                catch (InterruptedException ie) {
                }
                if (System.currentTimeMillis() - startTime > timeOutMillis) {
                    break;
                }
            }
        }
        return m_configured;
    }
     
	/////////////////////////////////////////////////////////
	/**
    * realize
	*/
	/////////////////////////////////////////////////////////

    public boolean realize(int timeOutMillis) 
    {
        long startTime = System.currentTimeMillis();
        synchronized (this) {
            m_player.realize();
            while (!m_realized && !m_failed) {
                try {
                    wait(timeOutMillis);
                }
                catch (InterruptedException ie) {
                }
                if (System.currentTimeMillis() - startTime > timeOutMillis) {
                    break;
                }
            }
        }
        return m_realized;
    }
 
	/////////////////////////////////////////////////////////
	/**
    * prefetch
	*/
	/////////////////////////////////////////////////////////

    public boolean prefetch(int timeOutMillis)
    {
        long startTime = System.currentTimeMillis();
        synchronized (this) {
            m_player.prefetch();
            while (!m_prefetched && !m_failed) {
                try {
                    wait(timeOutMillis);
                }
                catch (InterruptedException ie) {
                }
                if (System.currentTimeMillis() - startTime > timeOutMillis) {
                    break;
                }
            }
        }
        return m_prefetched && !m_failed;
    }

	/////////////////////////////////////////////////////////
	/**
    * playToEndOfMedia
	*/
	/////////////////////////////////////////////////////////
    
    public boolean playToEndOfMedia(int timeOutMillis) 
    {
        long startTime = System.currentTimeMillis();
        m_eom = false;
        synchronized (this) {
            m_player.start();
            while (!m_eom && !m_failed) {
                try {
                    wait(timeOutMillis);
                }
                catch (InterruptedException ie) {
                }
                if (System.currentTimeMillis() - startTime > timeOutMillis) {
                    break;
                }
            }
        }
        return m_eom && !m_failed;
    }
 
	/////////////////////////////////////////////////////////
	/**
    * close
	*/
	/////////////////////////////////////////////////////////

    public void close() 
    {
        synchronized (this) {
            m_player.close();
            while (!m_closed) {
                try {
                    wait(1000);
                }
                catch (InterruptedException ie) {
                }
            }
        }
        m_player.removeControllerListener(this);
    }
 
	/////////////////////////////////////////////////////////
	/**
    * controllerUpdate
	*/
	/////////////////////////////////////////////////////////

    public synchronized void controllerUpdate(ControllerEvent ce) 
    {
        System.err.println("ControllerUpdate : " + ce.toString());
        if (ce instanceof RealizeCompleteEvent) {
            m_realized = true;
        }
        else if (ce instanceof StartEvent) {
            m_recorder.trackMediaTime();
        }
        else if (ce instanceof ConfigureCompleteEvent) {
            m_configured = true;
        }
        else if (ce instanceof PrefetchCompleteEvent) {
            m_prefetched = true;
        }
        else if (ce instanceof EndOfMediaEvent) {
            m_eom = true;
        }
        else if (ce instanceof ControllerErrorEvent) {
            m_failed = true;
        }
        else if (ce instanceof ControllerClosedEvent) {
            m_closed = true;
        }
        else {
            return;
        }
        notifyAll();
    }

	/////////////////////////////////////////////////////////
	/**
	* setVolume
	*/
	/////////////////////////////////////////////////////////

    public void setVolume(float v) 
    {
        m_volume = v;
        if ( m_player != null) {
            GainControl gain = m_player.getGainControl();
            gain.setLevel(m_volume/100);
        }
    }
 }

 
