import java.io.*;
import java.util.*;
import java.net.InetAddress;
import javax.media.*;
import javax.media.control.*;
import javax.media.protocol.*;
import javax.media.format.*;
import javax.media.rtp.*;

public class rtpRecorder 
{
	/////////////////////////////////////////////////////////
	/**
    * member variables
	*/
	/////////////////////////////////////////////////////////

    private String m_serverName = null;
    private int m_cport = 0;
    private int m_sport = 0;
    private String m_url = null;
    private MediaLocator m_locator = null;
    private Processor m_processor = null;
    private rtpRecordListener m_sh = null;
    private DataSink m_sink = null;
    private DataSource m_dataOutput = null;
    private rtspClient m_client = null;
    private rtspControl m_cmd = null;
    private rtpRecordTimer m_timer = null;
    private String m_error = null;

	/////////////////////////////////////////////////////////
	/**
    * setURL
	*/
	/////////////////////////////////////////////////////////

    public rtpRecorder(rtspClient client)
    {
        m_client = client;
    }

	/////////////////////////////////////////////////////////
	/**
    * setURL
	*/
	/////////////////////////////////////////////////////////

    public void setURL( String url) 
    {
            m_url = url;
            int i=0;
            int start = 0;
            int end = 0;
            int p = m_url.indexOf(':',0);
            p = m_url.indexOf(':',p+1);
            i = m_url.indexOf('/',0);
            start = m_url.indexOf('/',i+1);
            end = m_url.indexOf('/',start+1);
            if ( p == -1) {
                m_serverName = m_url.substring(start+1,end);
            }
            else if ( p != -1 ) {
                m_serverName = m_url.substring(start+1,p);
            }
    }

	/////////////////////////////////////////////////////////
	/**
    * recordSession
	*/
	/////////////////////////////////////////////////////////

    public boolean recordSession()
    {
        try {
            if (m_cmd != null ) {
                m_cmd.closeSocket();
                m_cmd = null;
            }
            m_cmd = new rtspControl();
            m_cmd.setURL(m_url);
            m_cmd.openSocket();
            m_cmd.generateCSeq();
            m_cmd.generateClientPorts();

            // announce
            //m_cmd.rtspAnnounce();
            //if ( checkError() == false) {
            //    return false;
            //}
            
            // record setup
            m_cmd.rtspRecordSetup();
            if ( checkError() == false) {
                return false;
            }

            m_cport = m_cmd.getClientPort();
            m_sport = m_cmd.getServerPort();
            startRecording();
        }
        catch ( Exception e ) {
            m_error =  "Error: RTP Play Session : " + e.getMessage();
			System.out.println(m_error) ;
            m_client.outputSession(m_error);
            return false;
		}
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * stopRecordSession
	*/
	/////////////////////////////////////////////////////////

    public boolean stopRecordSession()
    {
        try {
            stopTrackMediaTime();
            m_sh.close();
            m_sink.close();
            m_cmd.rtspTeardown();
            m_cmd.closeSocket();
            m_sh = null;
            m_sink = null;
        }
        catch ( Exception ex) {
            m_error = "Error in stopSession : " + ex.toString();
            System.err.println(m_error);
            m_client.outputSession(m_error);
            return false;
        }
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * pauseSession
	*/
	/////////////////////////////////////////////////////////

    public boolean pauseSession()
    {
        try {
            pauseTrackMediaTime(true);
            m_cmd.rtspPause();
            m_sink.stop();
        }
        catch ( Exception ex) {
            m_error = "Error in pauseSession : " + ex.toString();
            System.err.println(m_error);
            m_client.outputSession(m_error);
            return false;
        }
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * unpauseSession
	*/
	/////////////////////////////////////////////////////////

    public boolean unpauseSession()
    {
        try {
            m_cmd.rtspRecord();
            if ( checkError() == false) {
                return false;
            }
            m_sink.start();
            pauseTrackMediaTime(false);
        }
        catch ( Exception ex) {
            m_error = "Error in unpauseSession : " + ex.toString();
            System.err.println(m_error);
            m_client.outputSession(m_error);
            return false;
        }
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * initMicrophone
	*/
	/////////////////////////////////////////////////////////

    private void initMicrophone() 
    {
        try {
            AudioFormat format = new AudioFormat(null);
            Vector deviceList=CaptureDeviceManager.getDeviceList(null);
            CaptureDeviceInfo captureInfo=null;
            Format deviceFormat[] = ((CaptureDeviceInfo)deviceList.elementAt(0)).getFormats();
            for(int j=0;j<deviceFormat.length;j++) {
                if(deviceFormat[j] instanceof AudioFormat) {
                    if(deviceFormat[j].matches(new AudioFormat(AudioFormat.LINEAR))) {
                        captureInfo=(CaptureDeviceInfo)deviceList.elementAt(0);
                        m_locator = captureInfo.getLocator();
                        System.err.println(captureInfo.toString());
                        break;
                     }
                }
            }
        }
        catch ( Exception ex) {
            m_error = "Error in initMicrophone : " + ex.toString();
            m_client.outputSession(m_error);
            System.err.println(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * setFormats
	*/
	/////////////////////////////////////////////////////////

    private boolean setFormats(Processor p) 
    {
        boolean found=false;
        try {
            TrackControl [] tracks = p.getTrackControls();
            Format rtpFormats[];
            for ( int i = 0; i < tracks.length; i++ ) {
                System.out.println( "Track #" + ( i + 1 ) + " supports " );
                if ( tracks[ i ].isEnabled() ) {
                    rtpFormats = tracks[ i ].getSupportedFormats();
                    if ( rtpFormats.length > 0 ) {
                        for ( int j = 0; j < rtpFormats.length; j++ ) {
                            System.out.println( rtpFormats[ j ] );
                            if (rtpFormats[j].toString().indexOf("ULAW/rtp",0) != -1) {
                                tracks[i].setFormat( rtpFormats[j] );
                                System.out.println ( "Track format set to " + tracks[i].getFormat() );
                                found = true;
                            }
                        }
                    }
                    else {
                        m_error = "No supported RTP formats for track!";
                        m_client.outputSession(m_error);
                        System.err.println (m_error);
                    }
                }
            } 
            if ( found == false) {
                 m_error = "Capture Device does not support ULAW/rtp";
                 m_client.outputSession(m_error);
                 System.out.println (m_error);
            }
        }
        catch (Exception ex) {
            m_error = "Error in setFormats : " + ex.toString();
            m_client.outputSession(m_error);
            System.err.println(m_error);
        }
        return found;
    }

	/////////////////////////////////////////////////////////
	/**
    * startRecording
	*/
	/////////////////////////////////////////////////////////

    public boolean startRecording() 
    {
        try {
            initMicrophone();
            createProcessor();
            createTransmitter();
            m_cmd.rtspRecord();
            if ( checkError() == false) {
                return false;
            }

            m_sink.start();
            m_processor.start();
        }
        catch (Exception ex) {
            m_processor.close();
            m_error = "Error in startRecording : " + ex.toString();
            m_client.outputSession(m_error);
            System.err.println(m_error);
        }
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * createProcessor
	*/
	/////////////////////////////////////////////////////////

    private boolean createProcessor() 
    {
        try {
            m_processor = Manager.createProcessor(m_locator);
            m_sh = new rtpRecordListener(m_processor, this);
            if (!m_sh.configure(10000)) {
                m_error = "Error configuring processor";
                m_client.outputSession(m_error);
                System.err.println(m_error);
                return false;
            }
            m_processor.setContentDescriptor(new FileTypeDescriptor(FileTypeDescriptor.RAW_RTP));
            setFormats(m_processor);
            if (!m_sh.realize(10000)) {
                m_processor.close();
                m_error = "Error realizing processor";
                m_client.outputSession(m_error);
                System.err.println(m_error);
                return false;
            }
        }
        catch (Exception ex) {
            m_error = "Error in createProcessor : " + ex.toString();
            m_client.outputSession(m_error);
            System.err.println(m_error);
        }
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * createTransmitter
	*/
	/////////////////////////////////////////////////////////
    
    public boolean createTransmitter() 
    {
        try {
            DataSource source = m_processor.getDataOutput();
            MediaLocator dest = null;
            String ip = InetAddress.getByName( m_serverName ).getHostAddress().toString();
            String url= "rtp://" + ip + ":" + m_sport + "/audio";
            dest = new MediaLocator(url);
            m_sink = Manager.createDataSink(source, dest);
            m_sink.open();
            StreamWriterControl swc = (StreamWriterControl) m_processor.getControl("javax.media.control.StreamWriterControl");
            if (swc != null) {
                swc.setStreamSizeLimit(500000);
            }
        }
        catch(Exception ex) {
            m_error = "Error in createTransmitter: " + ex.toString();
            m_client.outputSession(m_error);
            System.err.println(m_error);
            return false;
        }
        return true;
    }

	/////////////////////////////////////////////////////////
	/**
    * trackMediaTime
	*/
	/////////////////////////////////////////////////////////

    public void trackMediaTime()
    {
		try {
            if ( m_timer == null) {
                m_timer = new rtpRecordTimer(m_client);
                m_timer.start();
            }
		} 
        catch (Exception e) {
			System.err.println();
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * pauseTrackMediaTime
	*/
	/////////////////////////////////////////////////////////

    private void pauseTrackMediaTime(boolean pause)
    {
        try {
            if ( m_timer != null) {
                m_timer.pauseTimer(pause);
            }
        }
        catch (Exception e) {
			System.err.println("Error in stopTrackMediaTime :  " + e.toString());
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * stopTrackMediaTime
	*/
	/////////////////////////////////////////////////////////

    private void stopTrackMediaTime()
    {
        try {
            if ( m_timer != null) {
                m_timer.setDone();
                m_timer = null;
            }
        }
        catch (Exception e) {
			System.err.println("Error in stopTrackMediaTime :  " + e.toString());
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* checkError
	*/
	/////////////////////////////////////////////////////////

    private boolean checkError()
    {
        boolean res = true;
        try {
            String error = m_cmd.getError();
            if (error != null) {
                m_client.outputSession(error);
                m_cmd.closeSocket();
                m_cmd = null;
                res = false;
            }
        }
        catch ( Exception ex) {
			System.err.println("Error in CheckError");
        }
        return res;
    }

	/////////////////////////////////////////////////////////
	/**
	* setVolume
	*/
	/////////////////////////////////////////////////////////

    public void setVolume(float v) 
    {
        m_sh.setVolume(v);
    }
}
