import java.io.*;
import javax.swing.* ;
import javax.swing.event.*;

public class rtspClient extends javax.swing.JApplet implements ChangeListener
{
	/////////////////////////////////////////////////////////
	/**
    * member variables
	*/
	/////////////////////////////////////////////////////////

    // UI
    private JTextField m_subject_record = null;
    private JLabel m_subject_play = null;
    private JButton m_play = null;
    private JButton m_setup = null;
    private JButton m_pauseplay = null;
    private JButton m_record = null;
    private JButton m_pauserecord = null;
    private JButton m_stop = null;
    private JButton m_mute = null;
    private JLabel m_mediaTime = null;
    private JLabel m_mediaTimeLabel = null;
    private JLabel m_duration = null;
    private JSlider m_slider = null;
    private JSlider m_volume = null;
    public JTextArea m_output = null;
    private boolean m_movingSlider = false;
    private boolean m_muted = false;
    private String m_url = "";
    private String m_filename = "";
    private String m_rtsphost = "";
    private String m_rtspport = "";
    private String m_sip = "";
    private String m_email = "";
    private String m_dir = "";
    private String m_user = "";
    private String m_playrecord = "";

    private int m_state = STATE_STOPPED;
    private static int STATE_STOPPED = 0;
    private static int STATE_SETUP = 1;
    private static int STATE_PLAYING = 2;
    private static int STATE_PLAYING_PAUSED = 3;
    private static int STATE_RECORDING = 4;
    private static int STATE_RECORDING_PAUSED = 5;
    
    // players
    private rtpPlayer m_rtpPlayer = null;
    private rtpRecorder m_rtpRecorder = null;

	/////////////////////////////////////////////////////////
	/**
    * constructor
	*/
	/////////////////////////////////////////////////////////

    public rtspClient() {
    }

	/////////////////////////////////////////////////////////
	/**
    * init
	*/
	/////////////////////////////////////////////////////////

    public void init()
    {
        try {
            m_filename = this.getParameter("filename");
            m_rtsphost = this.getParameter("rtsphost");
            m_rtspport = this.getParameter("rtspport");
            m_sip = this.getParameter("sip");
            m_email = this.getParameter("email");
            m_dir = this.getParameter("dir");
            m_playrecord = this.getParameter("playrecord");
            buildUI();
            updateUI();
            m_rtpPlayer = new rtpPlayer(this);
            m_rtpRecorder = new rtpRecorder(this);
        }
		catch ( Exception e) {
			System.err.println("Error in rtspClient contruction : " + e.toString());
			System.exit(1);
		}
    }

	/////////////////////////////////////////////////////////
	/**
    * updateStoppedState
	*/
	/////////////////////////////////////////////////////////
    
    public void updateStoppedState()
    {
        m_setup.setVisible(true);
        m_setup.setEnabled(true);
        m_play.setVisible(false);
        m_pauseplay.setVisible(false);
        m_record.setVisible(true);
        m_record.setEnabled(true);
        m_pauserecord.setVisible(false);
        m_stop.setVisible(true);
        m_stop.setEnabled(false);
        m_mute.setEnabled(false);
        m_mute.setText("Mute On");
        m_muted = false;
        m_mediaTime.setText("0.0");
        m_duration.setText("");
        m_slider.setVisible(false);
    }

	/////////////////////////////////////////////////////////
	/**
    * updateSetupState
	*/
	/////////////////////////////////////////////////////////

    public void updateSetupState()
    {
        double duration = m_rtpPlayer.getDurationSecs();
        int i = (int) (duration * 10);
        double f = (double)(i/ 10.0);
        Double d = new Double(f);
        m_duration.setText("Total Time : " + d.toString());
        m_slider.setVisible(true);
        m_slider.setValue(0);
        m_slider.setMaximum( (int) (duration*10) );
        int tick = (int) (duration / 12.0);
        if (tick == 0) {
            tick=1;
        }
        m_slider.setMinorTickSpacing(tick);
        m_slider.setPaintTicks(true);
        m_duration.setVisible(true);
        m_setup.setVisible(false);
        m_play.setVisible(true);
    }

	/////////////////////////////////////////////////////////
	/**
    * updatePlayState
	*/
	/////////////////////////////////////////////////////////

    public void updatePlayState()
    {
        m_record.setEnabled(false);
        m_play.setVisible(false);
        m_pauseplay.setVisible(true);
        m_stop.setEnabled(true);
        m_mute.setEnabled(true);
        m_muted = false;
    }

	/////////////////////////////////////////////////////////
	/**
    * updatePlayPausedState
	*/
	/////////////////////////////////////////////////////////

    public void updatePlayPausedState()
    {
        m_pauseplay.setVisible(false);
        m_play.setVisible(true);
        m_stop.setEnabled(false);
        m_mute.setEnabled(false);
    }

	/////////////////////////////////////////////////////////
	/**
    * updateRecordState
	*/
	/////////////////////////////////////////////////////////

    public void updateRecordState()
    {
        updateStoppedState();
        m_setup.setEnabled(false);
        m_record.setEnabled(false);
        m_record.setEnabled(false);
        m_stop.setEnabled(true);
        m_mute.setEnabled(false);
    }

	/////////////////////////////////////////////////////////
	/**
    * updateRecordPausedState
	*/
	/////////////////////////////////////////////////////////

    public void updateRecordPausedState()
    {
        m_pauserecord.setVisible(false);
        m_record.setVisible(true);
        m_stop.setEnabled(false);
    }

	/////////////////////////////////////////////////////////
	/**
    * updateUI
	*/
	/////////////////////////////////////////////////////////

    public void updateUI()
    {
        if ( m_state == STATE_STOPPED) {
            updateStoppedState();
        }
        else if ( m_state == STATE_SETUP) {
            updateSetupState();
        }
        else if ( m_state == STATE_PLAYING) {
            updatePlayState();
        }
        else if ( m_state == STATE_PLAYING_PAUSED) {
            updatePlayPausedState();
        }
        else if ( m_state == STATE_RECORDING) {
            updateRecordState();
        }
        else if ( m_state == STATE_RECORDING_PAUSED) {
            updateRecordPausedState();
        }
     }

	/////////////////////////////////////////////////////////
	/**
    * setupSession
	*/
	/////////////////////////////////////////////////////////

    public void setupSession()
    {
        try {
            m_url = "rtsp://" + m_rtsphost + ":" + m_rtspport + "/" + m_user + "/" + m_dir + "/" + m_filename;
            m_rtpPlayer.setURL( m_url);
            m_output.setText("");
            if (m_rtpPlayer.setupSession()==false) {
                return;
            }
            m_output.setText("");
              
            m_state = STATE_SETUP;
            updateUI();
        }
        catch ( Exception e ) {
			System.out.println( "Error: RTP setupSession : " + e.getMessage() ) ;
		}
    }

	/////////////////////////////////////////////////////////
	/**
    * stopSetupSession
	*/
	/////////////////////////////////////////////////////////

    public void stopSetupSession()
    {
        try {
            m_state = STATE_STOPPED;
            updateUI();
            m_rtpPlayer.stopSetupSession();
        }
        catch ( Exception e ) {
			System.out.println( "Error: RTP stopPlaySession : " + e.getMessage() ) ;
		}
    }


	/////////////////////////////////////////////////////////
	/**
    * playSession
	*/
	/////////////////////////////////////////////////////////

    public void playSession()
    {
        try {
            if (m_rtpPlayer.playSession(getStartTime())==false) {
                return;
            }
            m_state = STATE_PLAYING;
            updateUI();
        }
        catch ( Exception e ) {
			System.out.println( "Error: RTP playSession : " + e.getMessage() ) ;
		}
    }

	/////////////////////////////////////////////////////////
	/**
    * stopPlaySession
	*/
	/////////////////////////////////////////////////////////

    public void stopPlaySession()
    {
        try {
            if (m_state != STATE_PLAYING_PAUSED) {
                m_state = STATE_STOPPED;
                updateUI();
            }
        }
        catch ( Exception e ) {
			System.out.println( "Error: RTP stopPlaySession : " + e.getMessage() ) ;
		}
    }

	/////////////////////////////////////////////////////////
	/**
    * pausePlaySession
	*/
	/////////////////////////////////////////////////////////

    public void pausePlaySession()
    {
        try {
            m_state = STATE_PLAYING_PAUSED;
            updateUI();
            if (m_rtpPlayer.pauseSession()==false) {
                return;
            }
        }
        catch ( Exception e ) {
			System.out.println( "Error: RTP pausePlaySession : " + e.getMessage() ) ;
		}
    }


	/////////////////////////////////////////////////////////
	/**
    * recordSession
	*/
	/////////////////////////////////////////////////////////

    public void recordSession()
    {
        try {
            if ( m_state == STATE_RECORDING_PAUSED) {
                if(m_rtpRecorder.unpauseSession()==false) {
                    m_rtpRecorder.stopRecordSession();
                    m_state = STATE_STOPPED;
                    updateUI();
                    return;
                }
            }
            else {
                String filename = m_subject_record.getText();
                filename = filename.replace(' ','_');
                if ( m_dir.equals("") == false) {
                    m_url = "rtsp://" + m_rtsphost + ":" + m_rtspport + "/" + m_user + "/" + m_dir + "/" + filename + ".au";
                }
                else {
                    m_url = "rtsp://" + m_rtsphost + ":" + m_rtspport + "/" + m_user + "/" + filename + ".au";
                }
                m_rtpRecorder.setURL( m_url);
                if (m_rtpRecorder.recordSession()==false) {
                    m_rtpRecorder.stopRecordSession();
                    m_state = STATE_STOPPED;
                    updateUI();
                    return;
                }
            }
            m_state = STATE_RECORDING;
            updateUI();
        }
        catch ( Exception e ) {
			System.out.println( "Error: RTP recordSession : " + e.getMessage() ) ;
		}
    }


	/////////////////////////////////////////////////////////
	/**
    * stopRecordSession
	*/
	/////////////////////////////////////////////////////////

    public void stopRecordSession()
    {
        try {
            m_rtpRecorder.stopRecordSession();
            m_state = STATE_STOPPED;
            updateUI();
        }
        catch ( Exception e ) {
			System.out.println( "Error: RTP stopRecordSession : " + e.getMessage() ) ;
		}
    }

	/////////////////////////////////////////////////////////
	/**
    * pauseRecordSession
	*/
	/////////////////////////////////////////////////////////

    public void pauseRecordSession()
    {
        try {
            m_state = STATE_RECORDING_PAUSED;
            updateUI();
            m_rtpRecorder.pauseSession();
        }
        catch ( Exception e ) {
			System.out.println( "Error: RTP pauseRecordSession : " + e.getMessage() ) ;
		}
    }

	/////////////////////////////////////////////////////////
	/**
    * getStartTime
	*/
	/////////////////////////////////////////////////////////

    public String getStartTime()
    {
        return m_mediaTime.getText();
    }

	/////////////////////////////////////////////////////////
	/**
    * updateMediaTime
	*/
	/////////////////////////////////////////////////////////

    public void updateMediaTime( double seconds )
    {
        if (m_movingSlider==false) {
            m_slider.setValue( (int) (seconds*10) );
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * stateChanges
	*/
	/////////////////////////////////////////////////////////

    public void stateChanged(ChangeEvent e) 
    {
        try {
            JSlider source = (JSlider)e.getSource();
            double t = source.getValue();
            String s = new Double(t/10.0).toString();
            if (source == m_slider) {
                m_mediaTime.setText(s);
            }
            else if (source == m_volume) {
                m_rtpPlayer.setVolume((float)t);
            }
        }
        catch ( Exception ex) {
			System.out.println( "Error: stateChanged : " + ex.getMessage() ) ;
        }
    }

	/////////////////////////////////////////////////////////
	/**
    * outputSession
	*/
	/////////////////////////////////////////////////////////

    public void outputSession( String s) 
    {
        m_output.setText("");
        m_output.append( s);
    }

	/////////////////////////////////////////////////////////
	/**
    * buildUI
	*/
	/////////////////////////////////////////////////////////

    private void buildUI()
    {
        if ( m_playrecord.equals("play") == true) {
            m_subject_play = new javax.swing.JLabel();
        }
        else {
            m_subject_record = new javax.swing.JTextField();
        }

        m_setup = new javax.swing.JButton("Setup");
        m_play = new javax.swing.JButton("Play");
        m_pauseplay = new javax.swing.JButton("Pause");
        m_record = new javax.swing.JButton("Record");
        m_pauserecord = new javax.swing.JButton("Pause");
        m_stop = new javax.swing.JButton("Stop");
        m_mute = new javax.swing.JButton("Mute");
        m_mediaTime = new javax.swing.JLabel();
        m_mediaTimeLabel = new javax.swing.JLabel("Current Time : ");
        m_duration = new javax.swing.JLabel();

        m_slider = new JSlider(JSlider.HORIZONTAL);
        m_slider.addChangeListener(this);
        m_slider.setPaintTicks(false);
        m_slider.setPaintLabels(false);
        m_slider.setValue(0);
        m_slider.setVisible(false);

        m_volume = new JSlider(JSlider.VERTICAL);
        m_volume.addChangeListener(this);
        m_volume.setMajorTickSpacing(10);
        m_volume.setMinorTickSpacing(10);
        m_volume.setPaintTicks(true);
        m_volume.setPaintLabels(false);
        m_volume.setMaximum(100);
        m_volume.setValue(50);

        m_output = new JTextArea();
        m_output.setLineWrap(true);

        m_state = STATE_STOPPED;
        updateUI();

        m_setup.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed( java.awt.event.ActionEvent evt) {
                setupSession();
            }
        });

        m_play.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed( java.awt.event.ActionEvent evt) {
                playSession();
            }
        });

        m_pauseplay.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed( java.awt.event.ActionEvent evt) {
                pausePlaySession();
            }
        });

        m_record.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed( java.awt.event.ActionEvent evt) {
                recordSession();
            }
        });

        m_pauserecord.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed( java.awt.event.ActionEvent evt) {
                pauseRecordSession();
            }
        });

        m_stop.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed( java.awt.event.ActionEvent evt) {
                if ( m_state == STATE_PLAYING) {
                    m_rtpPlayer.stopPlaySession();
                }
                else if ( m_state == STATE_RECORDING) {
                    stopRecordSession();
                }
            }
        });

        m_mute.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed( java.awt.event.ActionEvent evt) {
                if (m_muted == true) {
                    m_mute.setText("Mute On");
                    m_muted=false;
                }
                else {
                    m_mute.setText("Mute Off");
                    m_muted=true;
                }
                m_rtpPlayer.mute(m_muted);
            }
        });

        m_slider.addMouseListener( new java.awt.event.MouseListener() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {}
            public void mousePressed(java.awt.event.MouseEvent evt) {
                m_movingSlider = true;
            }
            public void mouseEntered(java.awt.event.MouseEvent evt) {}
            public void mouseExited(java.awt.event.MouseEvent evt) {}
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                if ( m_state == STATE_PLAYING) {
                    pausePlaySession();
                    playSession();
                }
                m_movingSlider = false;
            }
        });

        
        if ( m_playrecord.equals("play") == true) {
            m_subject_play.setBounds(10,10,400,25);
            m_subject_play.setText(m_filename);
        }
        else {
            m_subject_record.setBounds(10,10,400,25);
            m_subject_record.setText(m_filename);
        }

        int i = m_sip.indexOf("@");
        if (i!=-1) {
            m_user = m_sip.substring(0,i);
        }

        int buttWidth = 82;
        int buttHeight = 22;
        
        int xpos = 0;
        int ypos = 0;

        xpos = 10;
        ypos = 40;
        if ( m_playrecord.equals("play") == true) {
            m_setup.setBounds(xpos,ypos,buttWidth,buttHeight);
            m_play.setBounds(xpos,ypos,buttWidth,buttHeight);
            m_pauseplay.setBounds(xpos,ypos,buttWidth,buttHeight);
        }
        else {
            m_record.setBounds(xpos,ypos,buttWidth,buttHeight);
            m_pauserecord.setBounds(xpos,ypos,buttWidth,buttHeight);
        }
        xpos += buttWidth;
        m_stop.setBounds(xpos,ypos,buttWidth,buttHeight);
        xpos += buttWidth;
        m_mute.setBounds(xpos,ypos,buttWidth,buttHeight);

        xpos = 10;
        ypos += 30;
        m_mediaTimeLabel.setBounds(xpos,ypos,100,25);
        xpos += 85;
        m_mediaTime.setBounds(xpos,ypos,100,25);
        xpos += 50;
        m_duration.setBounds(xpos,ypos,100,25);

        xpos=10;
        ypos += 20;
        m_slider.setBounds(xpos,ypos,330,50);

        xpos = 10;
        ypos += 50;
        m_output.setBounds(xpos,ypos,400,40);

        xpos = 420;
        ypos = 10;
        m_volume.setBounds(xpos,ypos,50,140);

        getContentPane().setLayout( null);
        getContentPane().removeAll();
        if ( m_playrecord.equals("play") == true) {
            getContentPane().add( m_subject_play);
        }
        else {
            getContentPane().add( m_subject_record);
        }
        getContentPane().add( m_setup);
        getContentPane().add( m_play);
        getContentPane().add( m_pauseplay);
        getContentPane().add( m_record);
        getContentPane().add( m_pauserecord);
        getContentPane().add( m_stop);
        getContentPane().add( m_mute);
        getContentPane().add( m_mediaTimeLabel);
        getContentPane().add( m_mediaTime);
        getContentPane().add( m_duration);
        getContentPane().add( m_slider);
        getContentPane().add( m_volume);
        getContentPane().add( m_output);
    }
}
