import java.io.*;
import java.net.*;
import javax.media.rtp.*;

public class rtspControl {

	/////////////////////////////////////////////////////////
	/**
	* member variables
	*/
	/////////////////////////////////////////////////////////

	private String m_host = "localhost";        // host 
	private int m_rtsp_port = 8554;             // server port
	private Socket m_socket = null;             // socket for rtsp server
	private BufferedWriter m_output = null;     // writer for socket
	private BufferedReader m_input = null;      // reader for socket
    private String m_res;                       // Destination resource
    private String m_url;                       // rtsp url
    private String m_sess;                      // server session response
    private int m_cportl;                       // client low port
    private int m_cporth;                       // client high port
    private int m_sportl;                       // server low port
    private int m_sporth;                       // server high port
    private int m_cseq;                         // RTSP Sequence number
    private int m_rspt;                         // resource's play time
    private long m_durationMillisecs;           // size of the requested resource (in ms)
    private double m_durationSecs;              // size of the requested resource (in secs)
    private String m_error = null;
    private InetAddress m_local_addr = null;

	/////////////////////////////////////////////////////////
	/**
	* constructor
	*/
	/////////////////////////////////////////////////////////

    public rtspControl() {
    }

	/////////////////////////////////////////////////////////
	/**
	* setURL
	*/
	/////////////////////////////////////////////////////////

    public void setURL( String url)
    {
        try {
            m_url = url;
            int i=0;
            int start = 0;
            int end = 0;
            int p = m_url.indexOf(':',0);
            p = m_url.indexOf(':',p+1);
            i = m_url.indexOf('/',0);
            start = m_url.indexOf('/',i+1);
            end = m_url.indexOf('/',start+1);
            if ( p == -1) {
                m_host = m_url.substring(start+1,end);
            }
            else if ( p != -1 ) {
                m_host = m_url.substring(start+1,p);
                String s = m_url.substring(p+1,end);
                Integer t = new Integer(s);
                m_rtsp_port = t.intValue();
            }
        }
		catch ( Exception e) {
			System.err.println("Error in setURL");
		}
    }

	/////////////////////////////////////////////////////////
	/**
	* openSocket
	*/
	/////////////////////////////////////////////////////////

	public void openSocket()
	{
		try {
			m_socket = new Socket(m_host, m_rtsp_port);
			m_input = new BufferedReader( new InputStreamReader( m_socket.getInputStream()));
			m_output = new BufferedWriter( new OutputStreamWriter( m_socket.getOutputStream()));
            m_local_addr = m_socket.getLocalAddress();
		}
		catch ( Exception e) {
            m_error = "Error opening socket at host : " + m_host + ":" + m_rtsp_port;
			System.err.println(m_error);
		}
	}

	/////////////////////////////////////////////////////////
	/**
	* closeSocket
	*/
	/////////////////////////////////////////////////////////

	public void closeSocket()
	{
		try {
            if (m_socket != null) {
                m_input.close();
                m_output.close();
                m_socket.close();
                m_socket = null;
            }
			m_output = null;
			m_input = null;
		}
		catch (Exception e) {
            m_error = "Error closing socket at host : " + m_host + ":" + m_rtsp_port;
			System.err.println(m_error);
		}
	}

	/////////////////////////////////////////////////////////
	/**
	* readResponseLine
	*/
	/////////////////////////////////////////////////////////

    private String readResponseLine()
    {
        String response = "";
        try {
            response = m_input.readLine();
            System.err.println(response);
        }
        catch(Exception ex) {
            System.err.println(ex.toString());
        }
        return response;
    }

	/////////////////////////////////////////////////////////
	/**
	* writeRequest
	*/
	/////////////////////////////////////////////////////////

    private void writeRequest( String request )
    {
        try {
            m_output.write(request, 0, request.length());
            m_output.flush();
            System.err.println(request);
        }
        catch( Exception ex) {
            System.err.println(ex.toString());
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* generateCSeq
	*/
	/////////////////////////////////////////////////////////
    
    public void generateCSeq()
    {
        m_cseq = (int) (Math.random() * 10000);
    }

	/////////////////////////////////////////////////////////
	/**
	* generateClientPorts
	*/
	/////////////////////////////////////////////////////////
    
    public void generateClientPorts()
    {
        m_cportl = (((int) (Math.random() * 3000)) * 2) + 5000;
        m_cporth = m_cportl + 1;
    }

	/////////////////////////////////////////////////////////
	/**
	* rtspDescribe
	*/
	/////////////////////////////////////////////////////////

    public void rtspDescribe() 
    {
        try {
            m_cseq ++;
            String request = "DESCRIBE " + m_url + " RTSP/1.0\r\n";
            request += "CSeq: " + m_cseq + "\r\n";
            request += "Accept: audio/basic, audio/x-wav, application/sdp\r\n";
            request += "\r\n";
            writeRequest(request);
            int bytesTotal = 0;
            int bytesRead = 0;
            String response = readResponseLine();
            checkStatus(response, "DESCRIBE");
            while (response.equals("") == false ) {
                response = readResponseLine();
                if (response.startsWith("Content-Length: ")) {
                    bytesTotal = Integer.parseInt(response.substring(response.indexOf(": ")+2));
                }
            }

            int i=0;
            while (bytesRead < bytesTotal) {
                response = readResponseLine();
                bytesRead += response.length() + 2; 
                i = response.indexOf("npt=");
                if (i != -1) {
                    m_durationSecs = Double.valueOf(response.substring(i+4)).floatValue();
                    m_durationMillisecs = (long) m_durationSecs * 1000;
                }
            }
            System.err.println("rtspDescribe successful - duration = " + m_durationSecs  + "\n\n");
        }
        catch (Exception e) {
            m_error = "Error processing DESCRIBE request";
            e.printStackTrace();
            System.err.println(e.toString());
			System.err.println(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* rtspPlaySetup
	*/
	/////////////////////////////////////////////////////////

	public void rtspPlaySetup()
	{
		try {
            m_cseq ++;
            String request = "SETUP " + m_url + " RTSP/1.0\r\n";
            request += "CSeq: " + m_cseq + "\r\n";
            request += "Transport: RTP/AVP;unicast;client_port=" + m_cportl + "-" + m_cporth + "\r\n";
            request += "\r\n";
            writeRequest(request);

            String response = readResponseLine();
            checkStatus(response, "SETUP");
            int fld=0;
            while (!response.equals("")) {
                response = readResponseLine();
                if ((fld = response.indexOf("server_port=")) != -1) {
                    String sports = response.substring(response.indexOf("server_port=")+12);
                    if (sports.indexOf(";") > 0) {
                        sports = sports.substring(0, sports.indexOf(";"));
                    }
                    m_sportl = Integer.parseInt(sports.substring(0, sports.indexOf("-")));
                    m_sporth = Integer.parseInt(sports.substring(sports.indexOf("-")+1));
                    System.err.println("Server Port=" + m_sportl + "," + m_sporth);
                }
                else if ((fld = response.indexOf("Session: ")) != -1) {
                    String sess = response.substring(fld + 9);
                    m_sess = sess.substring(0, sess.indexOf(";"));
                }
            }
            System.err.println("rtspSetup successful\n\n");
		}
		catch (Exception e) {
            m_error = "Error processing SETUP request";
            e.printStackTrace();
            System.err.println(e.toString());
			System.err.println(m_error);
		}
	}

    /////////////////////////////////////////////////////////
	/**
	* rtspPlay
	*/
	/////////////////////////////////////////////////////////

    public void rtspPlay(String start)
    {
        try {
            m_cseq ++;
            String request = "PLAY " + m_url + " RTSP/1.0\r\n";
            request += "CSeq: " + m_cseq + "\r\n";
            request += "Session: " + m_sess  + "\r\n";
            request += "Range: npt=" + start + "-\r\n";
            request += "\r\n";
            writeRequest(request);

            String response = readResponseLine();
            checkStatus(response, "PLAY");
            while (!response.equals("")) {
                response = readResponseLine();
            }
            System.err.println("rtspPlay successful\n\n");
        }
        catch (Exception e) {
            m_error = "Error processing PLAY request";
            e.printStackTrace();
            System.err.println(e.toString());
			System.err.println(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* rtspAnnounce
	*/
	/////////////////////////////////////////////////////////

    public void rtspAnnounce()
    {
        try {
            m_cseq ++;
            String sdp = "v=0\r\n";
            sdp += "o=audio 12345678 12345679 IN IP4 127.0.0.1\r\n";
            sdp += "s=Recording Session\r\n";
            sdp += "a=control:" + m_url + "\r\n";
            sdp += "m=audio " + m_cportl + " RTP/AVP 0\r\n";
            sdp += "c=IN IP4 127.0.0.1\r\n";

            String request = "ANNOUNCE " + m_url + " RTSP/1.0\r\n";
            request += "CSeq: " + m_cseq + "\r\n";
            request += "Content-Type: application/sdp\r\n";
            request += "Content-Length: " + sdp.length() + "\r\n";
            request += "\r\n";
            request += sdp;
            writeRequest(request);

            String response = readResponseLine();
            checkStatus(response, "ANNOUNCE");
            while (!response.equals("")) {
                response = readResponseLine();
            }
            System.err.println("rtspAnnounce successful\n\n");
        }
        catch (Exception e) {
            m_error = "Error processing ANNOUNCE request";
            e.printStackTrace();
            System.err.println(e.toString());
			System.err.println(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* rtspRecordSetup
	*/
	/////////////////////////////////////////////////////////

	public void rtspRecordSetup()
	{
		try {
            m_cseq ++;
            String request = "SETUP " + m_url + " RTSP/1.0\r\n";
            request += "CSeq: " + m_cseq + "\r\n";
            request += "Transport: RTP/AVP;unicast;port=" + m_cportl + "-" + m_cporth + ";mode=record\r\n";
            request += "\r\n";
            writeRequest(request);
            String response = readResponseLine();
            checkStatus(response, "SETUP");
            int fld=0;
            while (!response.equals("")) {
                response = readResponseLine();
                if ((fld = response.indexOf("server_port=")) != -1) {
                    String sports = response.substring(response.indexOf("server_port=")+12);
                    if (sports.indexOf(";") > 0) {
                        sports = sports.substring(0, sports.indexOf(";"));
                    }
                    m_sportl = Integer.parseInt(sports.substring(0, sports.indexOf("-")));
                    m_sporth = Integer.parseInt(sports.substring(sports.indexOf("-")+1));
                    System.err.println("Server Port=" + m_sportl + "," + m_sporth);
                }
                else if ((fld = response.indexOf("Session: ")) != -1) {
                    String sess = response.substring(fld + 9);
                    m_sess = sess.substring(0, sess.indexOf(";"));
                }
            }
            System.err.println("rtspSetup successful\n\n");
		}
		catch (Exception e) {
            m_error = "Error processing SETUP request";
            e.printStackTrace();
            System.err.println(e.toString());
			System.err.println(m_error);
		}
	}

	/////////////////////////////////////////////////////////
	/**
	* rtspRecord
	*/
	/////////////////////////////////////////////////////////

    public void rtspRecord()
    {
        try {
            m_cseq ++;
            String request = "RECORD " + m_url + " RTSP/1.0\r\n";
            request += "CSeq: " + m_cseq + "\r\n";
            request += "Session: " + m_sess  + "\r\n";
            request += "\r\n";
            writeRequest(request);

            String response = readResponseLine();
            checkStatus(response, "RECORD");
            while (!response.equals("")) {
                response = readResponseLine();
            }
            System.err.println("rtspRecord successful\n\n");
        }
        catch (Exception e) {
            m_error = "Error processing RECORD request";
            e.printStackTrace();
            System.err.println(e.toString());
			System.err.println(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* rtspPause
	*/
	/////////////////////////////////////////////////////////

    public void rtspPause()
    {
        try {
            m_cseq ++;
            String request = "PAUSE " + m_url + " RTSP/1.0\r\n";
            request += "CSeq: " + m_cseq + "\r\n";
            request += "Session: " + m_sess  + "\r\n";
            request += "\r\n";
            writeRequest(request);

            String response = readResponseLine();
            checkStatus(response, "PAUSE");
            while (!response.equals("")) {
                response = readResponseLine();
            }
            System.err.println("rtspPause successful\n\n");
        }
        catch (Exception e) {
            m_error = "Error processing PLAY request";
            e.printStackTrace();
            System.err.println(e.toString());
			System.err.println(m_error);
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* rtspTeardown
	*/
	/////////////////////////////////////////////////////////

	public void rtspTeardown()
	{
		try {
            m_cseq ++;
            String request = "TEARDOWN " + m_url + " RTSP/1.0\r\n";
            request += "CSeq: " + m_cseq + "\r\n";
            request += "Session: " + m_sess  + "\r\n";
            request += "\r\n";
            writeRequest(request);

            String response = readResponseLine();
            checkStatus(response, "TEARDOWN");
            while (!response.equals("")) {
                response = readResponseLine();
            }
            System.err.println("rtspTeardown successful\n");
        }
		catch (Exception e) {
            m_error = "Error processing TEARDOWN request";
            e.printStackTrace();
            System.err.println(e.toString());
			System.err.println(m_error);
		}
    }

	/////////////////////////////////////////////////////////
	/**
	* checkStatus
	*/
	/////////////////////////////////////////////////////////

    private void checkStatus(String str, String action) 
    {
        String code = str.substring(str.indexOf(" ")+1,str.lastIndexOf(" "));
        if (!code.equals("200") && !code.equals("201")) {
            System.err.println("rtspclient: error : " + code + " during " + action);
            m_error = "RTSP error during " + action + " : " + str;
        }
    }

	/////////////////////////////////////////////////////////
	/**
	* getError
	*/
	/////////////////////////////////////////////////////////

    public String getError() 
    {
        return m_error;
    }

	/////////////////////////////////////////////////////////
	/**
	* getClientPort
	*/
	/////////////////////////////////////////////////////////

    public int getClientPort() 
    {
        return m_cportl;
    }

	/////////////////////////////////////////////////////////
	/**
	* getServerPort
	*/
	/////////////////////////////////////////////////////////

    public int getServerPort() 
    {
        return m_sportl;
    }

	/////////////////////////////////////////////////////////
	/**
	* getRtspPort
	*/
	/////////////////////////////////////////////////////////

    public int getRtspPort() 
    {
        return m_rtsp_port;
    }

	/////////////////////////////////////////////////////////
	/**
	* getLocalAddress
	*/
	/////////////////////////////////////////////////////////

    public InetAddress getLocalAddress() 
    {
        return m_local_addr;
    }

	/////////////////////////////////////////////////////////
	/**
	* getDurationSecs
	*/
	/////////////////////////////////////////////////////////

    public double getDurationSecs() 
    {
        return m_durationSecs;
    }

}
